@@ -403,7 +403,7 @@ set_os_spesific()
 	    mvv=''
 	;;
 
-	i[3456]86:Linux|x86_64:Linux|mips:Linux|ppc:Linux|sparc*:Linux)
+	i[3456]86:Linux|x86_64:Linux|mips:Linux|ppc:Linux|ppc64:Linux|sparc*:Linux)
 		cpf='-f'
 		if test "$verbose" -gt "1"
 		then
@@ -570,11 +570,25 @@ can_write_to()
 	    debug_msg 1 " ${test_folder}"
 	done
 
-	test -w "${test_folder}" || { debug_msg 1 "User \"${USERNAME}\" can't write to [${test_folder}]" ; return 1; }
-	debug_msg 1 "User \"${USERNAME}\" has write access to [${test_folder}]"
+	if test -w "${test_folder}"
+	then
+	    debug_msg 1 "User \"${USERNAME}\" has write access to [${test_folder}]"
+	else
+	    debug_msg 1 "User \"${USERNAME}\" can't write to [${test_folder}]"
+	    return 1
+	fi
     done
 }
 
+cannot_write_to()
+{
+    # Needed for the while loop in prompt_path because Solaris sh doesn't like ! as negation.
+    if can_write_to "$@"
+    then return 1
+    else return 0
+    fi
+}
+
 confirm_paths()
 {
     while true
@@ -634,7 +648,7 @@ prompt_path()
     read a_path
 
     test "${a_path}" || a_path="$2"
-    while ! can_write_to "${a_path}"
+    while cannot_write_to "${a_path}"
     do
 	echo "User \"${USERNAME}\" does not have write access to [${a_path}]."
 	if test "$1" = 'prefix'
@@ -781,7 +795,7 @@ generate_wrapper()
 	mips*:Linux)
 	;;
 
-	ppc:Linux)
+	ppc:Linux|ppc64:Linux)
 	    wrapper_ibmjava="
 	    IBMJava2-ppc-14/jre \\
 	    IBMJava2-ppc-131/jre \\"
@@ -802,6 +816,7 @@ export OPERA_BINARYDIR
 
 # Parse commandline parameters
 toset=
+_QTSTYLE_FROM_CMDLINE_='NO'
 for arg
 do
   if test \"\$toset\"
@@ -811,13 +826,19 @@ do
       toset=
   else
       case \$arg in
-  --binarydir*|-binarydir*) toset=OPERA_BINARYDIR ;;
-  --personaldir*|-personaldir*) toset=OPERA_PERSONALDIR ;;
+          --binarydir*|-binarydir*) toset=OPERA_BINARYDIR ;;
+          --personaldir*|-personaldir*) toset=OPERA_PERSONALDIR ;;
+          -style)
+              _QTSTYLE_FROM_CMDLINE_='YES'
+              ;;
       esac
   fi
 done
 test -z \"\$toset\" || die Missing parameter for \$arg
 
+test \"\${_QTSTYLE_FROM_CMDLINE_}\" = 'NO' && test -f \"\$HOME/.qt/qtrc\" && _OPERA_QTSTYLE=\`sed -n 's/^style=//p' \$HOME/.qt/qtrc\` && test \"\$_OPERA_QTSTYLE\" && set -- \"\$@\" -style \"\$_OPERA_QTSTYLE\"
+
+
 # Add ending path divider \"/\" if missing, so execution won't fail
 if test \`echo \${OPERA_BINARYDIR} | sed -e 's/.*[^\/]$/NO/'\` = 'NO'; then
 OPERA_BINARYDIR=\"\${OPERA_BINARYDIR}/\"
@@ -875,10 +896,14 @@ if test ! \"\${OPERA_JAVA_DIR}\"; then
 	/opt\"
 
     for SUNJAVA in \\
+	java-1.5.0-sun-1.5.0.06 \\
+	java-1.5.0-sun-1.5.0.06/jre \\
 	java-1.5.0-sun-1.5.0.05 \\
 	java-1.5.0-sun-1.5.0.05/jre \\
 	java-1.5.0-sun-1.5.0.04 \\
 	java-1.5.0-sun-1.5.0.04/jre \\
+	jre1.5.0_06 \\
+	jdk1.5.0_06/jre \\
 	jre1.5.0_05 \\
 	jdk1.5.0_05/jre \\
 	jre1.5.0_04 \\
@@ -999,7 +1024,19 @@ for LIBASPELL_DIR in \\
     if test -f \"\${LIBASPELL_DIR}/libaspell.so.15\"; then
         LD_LIBRARY_PATH=\"\${LD_LIBRARY_PATH}:\${LIBASPELL_DIR}\"
     fi
-done
+done"
+
+case "${os}" in
+	FreeBSD|NetBSD|OpenBSD)
+wrapper_contain="${wrapper_contain}	
+
+# Make sure the compat libraries are found
+test -d /usr/local/lib/compat/ && LD_LIBRARY_PATH=\"\${LD_LIBRARY_PATH}:/usr/local/lib/compat/\"
+"
+	;;
+esac
+
+wrapper_contain="${wrapper_contain}
 
 # Running Opera
 exec \"\${OPERA_BINARYDIR}opera\" \"\$@\"
@@ -1232,40 +1269,41 @@ run_install()
     mkdir $mkdirv $mkdirp $plugin_dir/
     chmod $chmodv 755 $plugin_dir
 
-    if test -f plugins/operamotifwrapper
-    then
-	cp $cpv $cpf plugins/operamotifwrapper $plugin_dir/
-	chmod $chmodv 755 $plugin_dir/operamotifwrapper
+    if test -f plugins/operapluginwrapper; then
+	cp $cpv $cpf plugins/operapluginwrapper $plugin_dir/
+	chmod $chmodv 755 $plugin_dir/operapluginwrapper
 	plugin_support='yes'
-    fi
+    else
+	if test -f plugins/operamotifwrapper; then
+	    cp $cpv $cpf plugins/operamotifwrapper $plugin_dir/
+	    chmod $chmodv 755 $plugin_dir/operamotifwrapper
+	    plugin_support='yes'
+	fi
 
-    if test -f plugins/operamotifwrapper-1
-    then
-	cp $cpv $cpf plugins/operamotifwrapper-1 $plugin_dir/
-	chmod $chmodv 755 $plugin_dir/operamotifwrapper-1
-	plugin_support='yes'
-    fi
+	if test -f plugins/operamotifwrapper-1; then
+	    cp $cpv $cpf plugins/operamotifwrapper-1 $plugin_dir/
+	    chmod $chmodv 755 $plugin_dir/operamotifwrapper-1
+	    plugin_support='yes'
+	fi
 
-    if test -f plugins/operamotifwrapper-2
-    then
-	cp $cpv $cpf plugins/operamotifwrapper-2 $plugin_dir/
-	chmod $chmodv 755 $plugin_dir/operamotifwrapper-2
-	plugin_support='yes'
-    fi
+	if test -f plugins/operamotifwrapper-2; then
+	    cp $cpv $cpf plugins/operamotifwrapper-2 $plugin_dir/
+	    chmod $chmodv 755 $plugin_dir/operamotifwrapper-2
+	    plugin_support='yes'
+	fi
 
-    if test -f plugins/operamotifwrapper-3
-    then
-	cp $cpv $cpf plugins/operamotifwrapper-3 $plugin_dir/
-	chmod $chmodv 755 $plugin_dir/operamotifwrapper-3
-	plugin_support='yes'
-    fi
+	if test -f plugins/operamotifwrapper-3; then
+	    cp $cpv $cpf plugins/operamotifwrapper-3 $plugin_dir/
+	    chmod $chmodv 755 $plugin_dir/operamotifwrapper-3
+	    plugin_support='yes'
+	fi
 
-    if test -f plugins/operamotifwrapper-4
-    then
-	cp $cpv $cpf plugins/operamotifwrapper-4 $plugin_dir/
-	chmod $chmodv 755 $plugin_dir/operamotifwrapper-4
-	plugin_support='yes'
-    fi
+	if test -f plugins/operamotifwrapper-4; then
+	    cp $cpv $cpf plugins/operamotifwrapper-4 $plugin_dir/
+	    chmod $chmodv 755 $plugin_dir/operamotifwrapper-4
+	    plugin_support='yes'
+	fi
+    fi # operapluginwrapper
 
     if test -f plugins/libnpp.so
     then
@@ -1301,7 +1339,11 @@ run_install()
     if test -z "${OPERADESTDIR}"
     then
 	# System wide configuration files
-	config_dir="/etc"
+	case "${os}" in
+	    FreeBSD|OpenBSD|NetBSD)	config_dir="/usr/local/etc";;
+	    *)				config_dir="/etc";;
+	esac
+
 	if can_write_to "$config_dir"
 	then
 	    echo
@@ -1337,12 +1379,7 @@ run_install()
 
 	if test "${bool_icons}" -ne 0
 	then
-	    icons
-	    gnome
-	    kde 3
-	    kde 2
-	    kde1
-	    mandrake
+	    xdg
 	fi
 
     fi # OPERADESTDIR
@@ -1390,16 +1427,78 @@ opera group "Internet"'
 generate_desktop()
 {
     # arg1 = location
+    # arg2 = type
 
-    desktop_file="$1/opera.desktop"
+    desktop_file="${1}/opera.desktop"
+    desktop_contain='[Desktop Entry]'
 
-    desktop_contain='[Desktop Entry]
+    if test ${2}; then
+	if test "${2}" = "xdg"; then
+	    desktop_contain="${desktop_contain}
+Version=1.0
+TryExec=opera"
+	fi
+
+	desktop_contain="${desktop_contain}
+Encoding=UTF-8
+Name=Opera
+Name[af]=opera
+Name[eo]=Opero
+Name[zu]=I Opera
+GenericName=Web browser
+GenericName[bs]=Web preglednik
+GenericName[de]=Web-Browser
+GenericName[eo]=TTT-rigardilo
+GenericName[es]=Navegador web
+GenericName[et]=Veebibrauser
+GenericName[eu]=Web arakatzailea
+GenericName[fi]=WWW-selain
+GenericName[fr]=Un navigateur web
+GenericName[is]=Vafri
+GenericName[it]=Browser Web
+GenericName[nl]=webbrowser
+GenericName[nn]=Nettlesar
+GenericName[pt]=Navegador Web
+GenericName[pt_BR]=Navegador
+GenericName[ro]=Navigator de web
+GenericName[sl]=Spletni brskalnik
+GenericName[ven]=Buronza ya Webu
+GenericName[xh]=Umkhangeli Zincwadi Zokubhaliweyo
+GenericName[zu]=Umkhangeli zincwadi we Web
+Exec=opera %u
+Terminal=false"
+
+# Application is not a category, according to
+# http://standards.freedesktop.org/menu-spec/menu-spec-latest.html
+#
+# but I include it anyway to fix
+# http://bugs.gentoo.org/show_bug.cgi?id=90508
+#
+# Other possible Categories:
+# Email, Presentation, IRCClient, FileTransfer, P2P, News, WebDevelopment
+
+	if test "${2}" = "xdg"; then
+	    desktop_contain="${desktop_contain}
+Categories=Application;Qt;Network;WebBrowser;X-Ximian-Main;X-Ximian-Toplevel
+Icon=opera.png"
+	else
+	    desktop_contain="${desktop_contain}
+Icon=opera"
+	fi
+
+	desktop_contain="${desktop_contain}
+MimeType=text/html;text/xml;application/xhtml+xml"
+    else
+	desktop_contain="${desktop_contain}
 Name=Opera
-Comment=Web Browser
 Exec=opera
 Icon=opera.xpm
-Terminal=0
-Type=Application'
+Terminal=0"
+    fi
+
+    desktop_contain="${desktop_contain}
+Comment=Web Browser
+Type=Application"
 
     echo "${desktop_contain}" > ${desktop_file}
     chmod $chmodv 644 ${desktop_file}
@@ -1682,8 +1781,39 @@ mandrake()
     fi
 }
 
+xdg()
+{   # http://standards.freedesktop.org
+    if update-desktop-database --help >/dev/null 2>&1; then
+
+	for ICON_DIR in ${XDG_DATA_DIRS}/icons/hicolor /usr/share/pixmaps/hicolor; do test -d ${ICON_DIR} && break; done
+
+	if   test ! -d ${ICON_DIR}; then echo "Could not find icon installation directory, icons not installed." >&2
+	elif test ! -w ${ICON_DIR}; then echo "Directory \"${ICON_DIR}\" not writable by user \"${USER}\", icons not installed." >&2
+	else
+	    test -d ${ICON_DIR}/48x48/apps && test -w ${ICON_DIR}/48x48/apps && cp $cpv $share_dir/images/opera_48x48.png $DIR_HI/48x48/apps/opera.png
+	    test -d ${ICON_DIR}/32x32/apps && test -w ${ICON_DIR}/32x32/apps && cp $cpv $share_dir/images/opera_32x32.png $DIR_HI/32x32/apps/opera.png
+	    test -d ${ICON_DIR}/22x22/apps && test -w ${ICON_DIR}/22x22/apps && cp $cpv $share_dir/images/opera_22x22.png $DIR_HI/22x22/apps/opera.png
+	fi
+
+	for SHORTCUT_DIR in ${XDG_DATA_HOME}/applications /usr/local/share/applications /usr/share/applications; do test -d ${SHORTCUT_DIR} && break; done
+
+	if   test ! -d ${SHORTCUT_DIR}; then echo "Could not find shortcut installation directory, desktop entry not installed." >&2; return
+	elif test ! -w ${SHORTCUT_DIR}; then echo "Directory \"${SHORTCUT_DIR}\" not writable by user \"${USER}\", desktop entry not installed." >&2; return
+	fi
+	generate_desktop ${SHORTCUT_DIR} xdg
+	update-desktop-database
+    else
+	icons
+	gnome
+	kde 3
+	kde 2
+	kde1
+	mandrake
+    fi
+}
+
 # These are the only variables the needs to be changed
-opera_version='8.51-20051114.5'
+opera_version='8.52-20060201.5'
 
 debug_level='0' # From 0 to 2
 
